# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-11-04 00:15
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('useragents', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Path',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(max_length=400)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.RenameField(
            model_name='visit',
            old_name='date',
            new_name='date_created',
        ),
        migrations.AddField(
            model_name='ip',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='useragent',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='visit',
            name='path',
            field=models.CharField(default='', max_length=400),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-11-05 21:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('useragents', '0008_auto_20171104_1731'),
    ]

    operations = [
        migrations.AddField(
            model_name='visit',
            name='content_type',
            field=models.CharField(blank=True, max_length=256),
        ),
        migrations.AddField(
            model_name='visit',
            name='encoding',
            field=models.CharField(blank=True, max_length=265),
        ),
        migrations.AddField(
            model_name='visit',
            name='method',
            field=models.CharField(blank=True, max_length=10),
        ),
    ]



import logging
logger = logging.getLogger(__name__)

from django import forms

from .models import Visit, Ip, Path, UserAgent

from django.core.exceptions import ObjectDoesNotExist


class IpForm(forms.ModelForm):
	
	class Meta:
		model = Ip
		fields = ['address',]
		
def add_ip(address):
	try:
		ip = Ip.objects.get(address=address)
		if ip:
			ip.save()
	except ObjectDoesNotExist:
		form = IpForm({'address':address})
		if form.is_valid():
			ip = form.save()
		else:
			logger.info(form.errors)
			ip = None
	return ip.id
		
class PathForm(forms.ModelForm):
	
	class Meta:
		model = Path
		fields = ['path',]
		
def add_path(path):
	try:
		path = Path.objects.get(path=path)
		if path:
			path.save()
	except ObjectDoesNotExist:
		form = PathForm({'path':path})
		if form.is_valid():
			path = form.save()
		else:
			logger.info(form.errors)
			path = None
	return path.id

class UserAgentForm(forms.ModelForm):
	
	class Meta:
		model = UserAgent
		fields = ['string',]
		
def add_user_agent(string):
	try:
		agent = UserAgent.objects.get(string=string)
		if agent:
			agent.save()
	except ObjectDoesNotExist:
		form = UserAgentForm({'string':string})
		if form.is_valid():
			agent = form.save()
		else:
			logger.info(form.errors)
			agent = None
	return agent.id

class VisitForm(forms.ModelForm):
	
	class Meta:
		model=Visit
		fields=['useragent','path','ip','method','content_type','encoding']
		
def add_visit(useragent,path,ip,method=None,content_type=None,encoding=None):
	try:
		useragent_obj = add_user_agent(useragent)
		path_obj = add_path(path)
		ip_obj = add_ip(ip)
	except:
		return
	
	form = VisitForm({
		'useragent':useragent_obj,
		'path':path_obj,
		'ip':ip_obj,
		'method':method,
		'content_type':content_type,
		'encoding':encoding,
	})
	if form.is_valid():
		visit = form.save()
	else:
		logger.info(form.errors)
		visit = None
	return visit
	



from django.db import models

class UserAgent(models.Model):
	string = models.TextField(unique=True)
	visits = models.PositiveIntegerField(default=0,blank=True)
	created = models.DateTimeField(auto_now_add=True)
	modified = models.DateTimeField(auto_now=True)
	
	def __str__(self):
		return self.string
	
	@property
	def short_agent(self):
		if len(self.string) > 40:
			return "{}...".format(self.string[:40])
		else:
			return self.string
		
	def increment(self):
		self.visits += 1
		
	def save(self, *args, **kwargs):
		self.increment()
		super(UserAgent, self).save(*args, **kwargs)
	
class Ip(models.Model):
	address = models.CharField(max_length=50,unique=True)
	visits = models.PositiveIntegerField(default=0,blank=True)
	created = models.DateTimeField(auto_now_add=True)
	modified = models.DateTimeField(auto_now=True)
	
	def __str__(self):
		return self.address
		
	def increment(self):
		self.visits += 1
		
	def save(self, *args, **kwargs):
		self.increment()
		super(Ip, self).save(*args, **kwargs)
	
class Path(models.Model):
	path = models.CharField(max_length=400,unique=True)
	visits = models.PositiveIntegerField(default=0,blank=True)
	created = models.DateTimeField(auto_now_add=True)
	modified = models.DateTimeField(auto_now=True)
	
	def __str__(self):
		return self.path
	
	def increment(self):
		self.visits += 1
	
	@property
	def short_path(self):
		if len(self.path) > 40:
			return "{}...".format(self.path[:40])
		else:
			return self.path
		
	def save(self, *args, **kwargs):
		self.increment()
		super(Path, self).save(*args, **kwargs)

class Visit(models.Model):
	useragent = models.ForeignKey(UserAgent)
	ip = models.ForeignKey(Ip)
	path = models.ForeignKey(Path)
	method = models.CharField(max_length=10,blank=True)
	content_type = models.CharField(max_length=256,blank=True)
	encoding = models.CharField(max_length=265,blank=True)
	created = models.DateTimeField(auto_now_add=True)
	
	def __str__(self):
		return "{0} - {1} - {2}".format(
			self.useragent.short_agent, 
			self.ip, 
			self.path.short_path,
		)



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-11-04 19:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('useragents', '0004_auto_20171104_1239'),
    ]

    operations = [
        migrations.RenameField(
            model_name='ip',
            old_name='date_created',
            new_name='created',
        ),
        migrations.RenameField(
            model_name='path',
            old_name='date_created',
            new_name='created',
        ),
        migrations.RenameField(
            model_name='useragent',
            old_name='date_created',
            new_name='created',
        ),
        migrations.RenameField(
            model_name='visit',
            old_name='date_created',
            new_name='created',
        ),
        migrations.AddField(
            model_name='ip',
            name='modified',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AddField(
            model_name='path',
            name='modified',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AddField(
            model_name='useragent',
            name='modified',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AddField(
            model_name='visit',
            name='modified',
            field=models.DateTimeField(auto_now=True),
        ),
    ]



from django.contrib import admin

from .models import Ip, Visit, UserAgent, Path

@admin.register(Ip)
class IpAdmin(admin.ModelAdmin):
    list_display = ['address','visits','created','modified',]
    search_fields = ['address',]
    
@admin.register(Visit)
class VisitAdmin(admin.ModelAdmin):
    list_display = ['useragent','ip','path','method','content_type','encoding','created',]
    search_fields = ['useragent__string','ip__address','path__path','method','content_type','encoding']
    
@admin.register(Path)
class PathAdmin(admin.ModelAdmin):
    list_display = ['path','visits','created','modified',]
    search_fields = ['path',]
    
@admin.register(UserAgent)
class UserAgentAdmin(admin.ModelAdmin):
    list_display = ['string','visits','created','modified',]
    search_fields = ['string',]



import logging
logger = logging.getLogger(__name__)

from ipware.ip import get_real_ip

from useragents.forms import add_visit
	

def log_user_agent(request):
	user_agent = request.META.get('HTTP_USER_AGENT', '')
	ip = get_real_ip(request)
	if ip == None:
		ip = 'No IP Info'
	path = request.path
	content_type = request.content_type
	method = request.method
	encoding = request.encoding
	add_visit(user_agent,path,ip,method,content_type,encoding)
	
		
		
class UserAgentsMiddleware(object):

    def __init__(self, get_response=None):
        if get_response is not None:
            self.get_response = get_response

    def __call__(self, request):
        self.process_request(request)
        return self.get_response(request)

    def process_request(self, request):
        log_user_agent(request)



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-11-05 00:31
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('useragents', '0007_auto_20171104_1327'),
    ]

    operations = [
        migrations.AlterField(
            model_name='visit',
            name='path',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='useragents.Path'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-11-04 20:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('useragents', '0005_auto_20171104_1248'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='visit',
            name='modified',
        ),
        migrations.AlterField(
            model_name='ip',
            name='visits',
            field=models.PositiveIntegerField(blank=True, default=0, unique=True),
        ),
        migrations.AlterField(
            model_name='path',
            name='visits',
            field=models.PositiveIntegerField(blank=True, default=0, unique=True),
        ),
        migrations.AlterField(
            model_name='useragent',
            name='visits',
            field=models.PositiveIntegerField(blank=True, default=0, unique=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-11-03 23:58
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Ip',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip', models.CharField(max_length=50)),
            ],
        ),
        migrations.CreateModel(
            name='UserAgent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('string', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='Visit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateTimeField(auto_now_add=True)),
                ('ip', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='useragents.Ip')),
                ('useragent', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='useragents.UserAgent')),
            ],
        ),
    ]
